#include <Servo.h>

#define RED_LED 8
#define RED_LED_MASK 8
#define RED_LED_FLASH_MASK 128
#define BLUE_LED 10
#define BLUE_LED_MASK 4
#define BLUE_LED_FLASH_MASK 64
#define GREEN_LED 11
#define GREEN_LED_MASK 2
#define GREEN_LED_FLASH_MASK 32
#define YELLOW_LED 12
#define YELLOW_LED_MASK 1
#define YELLOW_LED_FLASH_MASK 16
#define GAUGE_NEEDLE 9

#define RANDOM_SEED 0
#define KNOB_ONE 1
#define KNOB_TWO 2
#define SYNCH_TARGET 40

#define GAUGE_MINIMUM 43
#define GAUGE_DEEP_MIN_THRESHOLD 60
#define GAUGE_MIN_THRESHOLD 77
#define GAUGE_DEEP_MAX_THRESHOLD 130
#define GAUGE_MAX_THRESHOLD 113
#define GAUGE_MAXIMUM 147

#define GAUGE_WIGGLE_DURATION_MIN 50
#define GAUGE_WIGGLE_DURATION_MAX 200

#define GAUGE_DURATION_MIN_ONE 1000
#define GAUGE_DURATION_MAX_ONE 7000
#define GAUGE_DURATION_MIN_TWO 2000
#define GAUGE_DURATION_MAX_TWO 5000

#define CONSTANT_FLASH_DURATION 600


#define STATE_SYNCH 0
#define STATE_MEASURE 1

Servo gauge_needle;

byte led_states;
unsigned long flash_start_green;
unsigned long flash_start_yellow;
unsigned long flash_start_blue;
unsigned long flash_start_red;
int knob_one_target, knob_two_target;
byte gauge_bias;
int gauge_value;
int deep_blue_count;
int blue_count;
int mid_count;
int red_count;
int deep_red_count;
long count_weight[5];

unsigned long gauge_duration;
unsigned long gauge_last_move;
unsigned long gauge_move_duration;
byte gauge_target;
unsigned long gauge_target_achieve_time;
bool achieved_target;
unsigned long current_time;
int state;
bool measure_reset_started;
bool synch_reset_started;

void greenLedOn()
{
  if ((led_states & GREEN_LED_MASK) == 0)
  {
    digitalWrite(GREEN_LED, HIGH);
    led_states = led_states + GREEN_LED_MASK;
  }
}

void greenLedOff()
{
  if ((led_states & GREEN_LED_MASK) > 0)
  {
    digitalWrite(GREEN_LED, LOW);
    led_states = led_states - GREEN_LED_MASK;
  }
}

void greenLedStartFlash()
{
  if ((led_states & GREEN_LED_FLASH_MASK) == 0)
  {
    greenLedOn();
    flash_start_green = current_time;
    led_states = led_states | GREEN_LED_FLASH_MASK;
  }
}

void greenLedEndFlash(bool is_on_after)
{
  if ((led_states & GREEN_LED_FLASH_MASK) > 0)
  {
    led_states = led_states - GREEN_LED_FLASH_MASK;
  }
  if (is_on_after)
  {
    greenLedOn();
  }
  else
  {
    greenLedOff();
  }
}

void greenLedFlash(unsigned long duration)
{
  if ((led_states & GREEN_LED_FLASH_MASK) > 0)
  {
    unsigned long elapsed_time = current_time - flash_start_green;
    if (elapsed_time >= duration)    
    {
      flash_start_green = current_time;
      if ((led_states & GREEN_LED_MASK) > 0)
      {
        digitalWrite(GREEN_LED, LOW);
        led_states = led_states - GREEN_LED_MASK;
      }
      else
      {
        digitalWrite(GREEN_LED, HIGH);
        led_states = led_states + GREEN_LED_MASK;
      }
    }
  }
}

void redLedOn()
{
  if ((led_states & RED_LED_MASK) == 0)
  {
    digitalWrite(RED_LED, HIGH);
    led_states = led_states + RED_LED_MASK;
  }
}

void redLedOff()
{
  if ((led_states & RED_LED_MASK) > 0)
  {
    digitalWrite(RED_LED, LOW);
    led_states = led_states - RED_LED_MASK;
  }
}

void redLedStartFlash()
{
  if ((led_states & RED_LED_FLASH_MASK) == 0)
  {
    redLedOn();
    flash_start_red = current_time;
    led_states = led_states | RED_LED_FLASH_MASK;
  }
}

void redLedEndFlash(bool is_on_after)
{
  if ((led_states & RED_LED_FLASH_MASK) > 0)
  {
    led_states = led_states - RED_LED_FLASH_MASK;
  }
  if (is_on_after)
  {
    redLedOn();
  }
  else
  {
    redLedOff();
  }
}

void redLedFlash(unsigned long duration)
{
  if ((led_states & RED_LED_FLASH_MASK) > 0)
  {
    unsigned long elapsed_time = current_time - flash_start_red;
    if (elapsed_time >= duration)    
    {
      flash_start_red = current_time;
      if ((led_states & RED_LED_MASK) > 0)
      {
        digitalWrite(RED_LED, LOW);
        led_states = led_states - RED_LED_MASK;
      }
      else
      {
        digitalWrite(RED_LED, HIGH);
        led_states = led_states + RED_LED_MASK;
      }
    }
  }
}

void blueLedOn()
{
  if ((led_states & BLUE_LED_MASK) == 0)
  {
    digitalWrite(BLUE_LED, HIGH);
    led_states = led_states + BLUE_LED_MASK;
  }
}

void blueLedOff()
{
  if ((led_states & BLUE_LED_MASK) > 0)
  {
    digitalWrite(BLUE_LED, LOW);
    led_states = led_states - BLUE_LED_MASK;
  }
}

void blueLedStartFlash()
{
  if ((led_states & BLUE_LED_FLASH_MASK) == 0)
  {
    blueLedOn();
    flash_start_blue = current_time;
    led_states = led_states | BLUE_LED_FLASH_MASK;
  }
}

void blueLedEndFlash(bool is_on_after)
{
  if ((led_states & BLUE_LED_FLASH_MASK) > 0)
  {
    led_states = led_states - BLUE_LED_FLASH_MASK;
  }
  if (is_on_after)
  {
    blueLedOn();
  }
  else
  {
    blueLedOff();
  }
}

void blueLedFlash(unsigned long duration)
{
  if ((led_states & BLUE_LED_FLASH_MASK) > 0)
  {
    unsigned long elapsed_time = current_time - flash_start_blue;
    if (elapsed_time >= duration)
    {
      flash_start_blue = current_time;
      if ((led_states & BLUE_LED_MASK) > 0)
      {
        digitalWrite(BLUE_LED, LOW);
        led_states = led_states - BLUE_LED_MASK;
      }
      else
      {
        digitalWrite(BLUE_LED, HIGH);
        led_states = led_states + BLUE_LED_MASK;
      }
    }
  }
}


void yellowLedOn()
{
  if ((led_states & YELLOW_LED_MASK) == 0)
  {
    digitalWrite(YELLOW_LED, HIGH);
    led_states = led_states + YELLOW_LED_MASK;
  }
}

void yellowLedOff()
{
  if ((led_states & YELLOW_LED_MASK) > 0)
  {
    digitalWrite(YELLOW_LED, LOW);
    led_states = led_states - YELLOW_LED_MASK;
  }
}

void yellowLedStartFlash()
{
  if ((led_states & YELLOW_LED_FLASH_MASK) == 0)
  {
    yellowLedOn();
    flash_start_yellow = current_time;
    led_states = led_states | YELLOW_LED_FLASH_MASK;
  }
}

void yellowLedEndFlash(bool is_on_after)
{
  if ((led_states & YELLOW_LED_FLASH_MASK) > 0)
  {
    led_states = led_states - YELLOW_LED_FLASH_MASK;
  }
  if (is_on_after)
  {
    yellowLedOn();
  }
  else
  {
    yellowLedOff();
  }
}

void yellowLedFlash(unsigned long duration)
{
  if ((led_states & YELLOW_LED_FLASH_MASK) > 0)
  {
    unsigned long elapsed_time = current_time - flash_start_yellow;
    if (elapsed_time >= duration)    
    {
      flash_start_yellow = current_time;
      if ((led_states & YELLOW_LED_MASK) > 0)
      {
        digitalWrite(YELLOW_LED, LOW);
        led_states = led_states - YELLOW_LED_MASK;
      }
      else
      {
        digitalWrite(YELLOW_LED, HIGH);
        led_states = led_states + YELLOW_LED_MASK;
      }
    }
  }
}

byte convertVoltRangeToGaugeBias(int volt_range)
{
  float float_volt = volt_range;
  float volt_max = 1023;
  float max_gauge_float = GAUGE_MAXIMUM;
  float min_gauge_float = GAUGE_MINIMUM;
  float result_as_float = ((max_gauge_float - min_gauge_float) * (float_volt / volt_max)) + min_gauge_float;
  int result = result_as_float;
  byte final_result = 2;
  if (result <= GAUGE_DEEP_MIN_THRESHOLD)
  {
    final_result = 0;
  }
  else if (result <= GAUGE_MIN_THRESHOLD)
  {
    final_result = 1;
  }
  else if (result >= GAUGE_DEEP_MAX_THRESHOLD)
  {
    final_result = 4; 
  }
  else if (result >= GAUGE_MAX_THRESHOLD)
  {
    final_result = 3;
  }
  return final_result;
}

void initializeSynch()
{
  state = STATE_SYNCH;
  redLedEndFlash(false);
  greenLedEndFlash(false);
  blueLedEndFlash(false);
  yellowLedStartFlash();
  int current_value = analogRead(KNOB_ONE);
  gauge_bias = convertVoltRangeToGaugeBias(current_value);
  knob_one_target = random(23, 1000);
  while (abs(knob_one_target - current_value) < 200)
  {
    knob_one_target = random(15, 1000);
  }
  current_value = analogRead(KNOB_TWO);
  knob_two_target = random(23, 1000);
  while (abs(knob_two_target - current_value) < 200)
  {
    knob_two_target = random(23, 1000);
  }
}

void updateZoneCounts()
{
  if (gauge_value <= GAUGE_DEEP_MIN_THRESHOLD)
  {
    if (deep_blue_count > 0)
    {
      deep_blue_count = -1;
    }
    else
    {
      deep_blue_count -= 1;
    }
    blue_count++;
    mid_count++;
    red_count++;
    deep_red_count++;
  }
  else if (gauge_value <= GAUGE_MIN_THRESHOLD)
  {
    if (blue_count > 0)
    {
      blue_count = -1;
    }
    else
    {
      blue_count -= 1;
    }
    deep_blue_count++;
    mid_count++;
    red_count++;
    deep_red_count++;
  }
  else if (gauge_value >= GAUGE_DEEP_MAX_THRESHOLD)
  {
    if (deep_red_count > 0)
    {
      deep_red_count = -1;
    }
    else
    {
      deep_red_count -= 1;
    }
    deep_blue_count++;
    blue_count++;
    mid_count++;
    red_count++;
  }
  else if (gauge_value >= GAUGE_MAX_THRESHOLD)
  {
    if (red_count > 0)
    {
      red_count = -1;
    }
    else
    {
      red_count -= 1;
    }
    deep_blue_count++;
    blue_count++;
    mid_count++;
    deep_red_count++;
  }
  else
  {
    if (mid_count > 0)
    {
      mid_count = -1;
    }
    else
    {
      mid_count -= 1;
    }
    deep_blue_count++;
    blue_count++;
    red_count++;
    deep_red_count++;
  }
}

void updateGaugeMoveDuration()
{
  gauge_move_duration = (random(GAUGE_WIGGLE_DURATION_MIN, GAUGE_WIGGLE_DURATION_MAX + 1) + random(GAUGE_WIGGLE_DURATION_MIN, GAUGE_WIGGLE_DURATION_MAX + 1)) / 2;
  gauge_last_move = current_time;
}
void updateGaugeDuration()
{
  gauge_duration = (random(GAUGE_DURATION_MIN_ONE, GAUGE_DURATION_MAX_ONE + 1L) + random(GAUGE_DURATION_MIN_ONE, GAUGE_DURATION_MAX_ONE + 1L) + random(GAUGE_DURATION_MIN_TWO, GAUGE_DURATION_MAX_TWO + 1L)) / 3;
}

void calculateInitialGaugeValue()
{
  
  deep_blue_count = 0;
  blue_count = 0;
  mid_count = 0;
  red_count = 0;
  deep_red_count = 0;
  gauge_target = gauge_bias;
  switch (gauge_target)
  {
    case 0:
      count_weight[0] = 4000L;
      count_weight[1] = 1000L;
      count_weight[2] = 100L;
      count_weight[3] = 10L;
      count_weight[4] = 1L;
    break;
    case 1:
      count_weight[0] = 100L;
      count_weight[1] = 400L;
      count_weight[2] = 100L;
      count_weight[3] = 10L;
      count_weight[4] = 1L;
      break;
    case 3:
      count_weight[4] = 100;
      count_weight[3] = 400;
      count_weight[2] = 100;
      count_weight[1] = 10;
      count_weight[0] = 1;
      break;
    case 4:
      count_weight[4] = 4000L;
      count_weight[3] = 1000L;
      count_weight[2] = 100L;
      count_weight[1] = 10L;
      count_weight[0] = 1L;
      break;
    default:
      count_weight[0] = 1L;
      count_weight[1] = 10L;
      count_weight[2] = 40L;
      count_weight[3] = 10L;
      count_weight[4] = 1L;
      break;
  }
  gauge_target_achieve_time = current_time;
  achieved_target = false;
  updateGaugeDuration();
  updateGaugeMoveDuration();
  switch(gauge_target)
  {
    case 0:
      gauge_value = random(GAUGE_MINIMUM + 1L, GAUGE_DEEP_MIN_THRESHOLD);
      break;
    case 1:
      gauge_value = random(GAUGE_DEEP_MIN_THRESHOLD + 1L, GAUGE_MIN_THRESHOLD);
      break;
    case 3:
      gauge_value = random(GAUGE_MAX_THRESHOLD + 1L, GAUGE_DEEP_MAX_THRESHOLD);
      break;
    case 4:
      gauge_value = random(GAUGE_DEEP_MAX_THRESHOLD + 1L, GAUGE_MAXIMUM);
      break;
    default:
      gauge_value = (random(GAUGE_MINIMUM + 1L, GAUGE_MAXIMUM) + random(GAUGE_MINIMUM + 1L, GAUGE_MAXIMUM)) / 2;
      break;
  }
  updateZoneCounts();
}

bool inZone(int &minRange, int &maxRange, bool &direction_up)
{
  switch (gauge_target)
  {
    case 0:
      if (gauge_value <= GAUGE_DEEP_MIN_THRESHOLD)
      {
          minRange = GAUGE_MINIMUM - gauge_value;         
          if (minRange < -8)
          {
            minRange = -8;
          }
          maxRange = GAUGE_DEEP_MIN_THRESHOLD - gauge_value;
          if (maxRange > 8)
          {
            maxRange = 8;
          }
          return true;
      }
      direction_up = false;
      break;
    case 1:
      if ((gauge_value > GAUGE_DEEP_MIN_THRESHOLD) && (gauge_value <= GAUGE_MIN_THRESHOLD))
      {
          minRange = (GAUGE_DEEP_MIN_THRESHOLD + 1) - gauge_value;
          if (minRange < -8)
          {
            minRange = -8;
          }
          maxRange = GAUGE_MIN_THRESHOLD - gauge_value;
          if (maxRange > 8)
          {
            maxRange = 8;
          }
          return true;
      }
      if (gauge_value <= GAUGE_DEEP_MIN_THRESHOLD)
      {
        direction_up = true;
      }
      else
      {
        direction_up = false;
      }
      break;
    case 3:
      if ((gauge_value >= GAUGE_MAX_THRESHOLD) && (gauge_value < GAUGE_DEEP_MAX_THRESHOLD))
      {
          minRange = GAUGE_MAX_THRESHOLD - gauge_value;
          if (minRange < -8)
          {
            minRange = -8;
          }
          maxRange = (GAUGE_DEEP_MAX_THRESHOLD - 1) - gauge_value;
          if (maxRange > 8)
          {
            maxRange = 8;
          }
          return true;
      }
      if (gauge_value < GAUGE_MAX_THRESHOLD)
      {
        direction_up = true;
      }
      else
      {
        direction_up = false;
      }
      break;
    case 4:
      if (gauge_value >= GAUGE_DEEP_MAX_THRESHOLD)
      {
        minRange = GAUGE_DEEP_MAX_THRESHOLD - gauge_value;
        if (minRange < -8)
        {
          minRange = -8;
        }
        maxRange = GAUGE_MAXIMUM - gauge_value;
        if (maxRange > 8)
        {
          maxRange = 8;
        }
        return true;
      }
      direction_up = true;
      break;
    default:
      if ((gauge_value > GAUGE_MIN_THRESHOLD) && (gauge_value < GAUGE_MAX_THRESHOLD))
      {
        minRange = (GAUGE_MIN_THRESHOLD + 1) - gauge_value;
        if (minRange < -8)
        {
          minRange = -8;
        }
        maxRange = (GAUGE_MAX_THRESHOLD - 1) - gauge_value;
        if (maxRange > 8)
        {
          maxRange = 8;
        }
        return true;
      }
      if (gauge_value <= GAUGE_MIN_THRESHOLD)
      {
        direction_up = true;
      }
      else
      {
        direction_up = false;
      }
      break;
  }
  minRange = GAUGE_MINIMUM - gauge_value;
  if (minRange < -8)
  {
    minRange = -8;
  }
  maxRange = GAUGE_MAXIMUM - gauge_value;
  if (maxRange > 8)
  {
    maxRange = 8;
  }
  return false;
}

long getChance(int count, long weight_rank)
{
  long result = 10;
  if (count < -1)
  {
    switch (count)
    {
      case -2:
        result = 8;
        break;
      case -3:
        result = 3;
        break;
      default:
        result = 0;
        break;
    }
  }
  else if (count > 6)
  {
    if (count > 61)
    {
      count = 61;
    }
    result = 20 * (10 ^ ((count - 6) / 20));
  }
  result = result * weight_rank;
  return result;
}


void generateNextTarget()
{
  long chances[5];
  chances[0] = getChance(deep_blue_count, count_weight[0]);
  chances[1] = getChance(blue_count, count_weight[1]);
  chances[2] = getChance(mid_count, count_weight[2]);
  chances[3] = getChance(red_count, count_weight[3]);
  chances[4] = getChance(deep_red_count, count_weight[4]);
  long total_chance;
  total_chance = chances[0] + chances[1] + chances[2] + chances[3] + chances [4];
  long roll = random(0, total_chance);
  if (roll < chances[0])
  {
    gauge_target = 0;
  }
  else
  {
    roll = roll - chances[0];
    if (roll < chances[1])
    {
      gauge_target = 1;
    }
    else
    {
      roll = roll - chances[1];
      if (roll < chances[2])
      {
        gauge_target = 2;
      }
      else
      {
        roll = roll - chances[2];
        if (roll < chances[3])
        {
          gauge_target = 3;
        }
        else
        {
          gauge_target = 4;
        }
      }
    }
  }
  achieved_target = false;
  updateGaugeDuration();
  updateZoneCounts();
}

int generateGaugeDelta()
{
  int minRange = -8;
  int maxRange = 8;
  bool direction_up = false;
  int result = 0;
  if (inZone(minRange, maxRange, direction_up))
  {
    if (achieved_target == false)
    {
      gauge_target_achieve_time = current_time;
      achieved_target = true;
    }
    else
    {
      unsigned long elapsed_time = current_time - gauge_target_achieve_time;
      if (elapsed_time > gauge_duration)
      {
        generateNextTarget();
      }
    }
    result = (random(minRange, maxRange + 1) + random(minRange, maxRange + 1)) / 2;
  }
  else
  {
    if (direction_up)
    {
      result = (random(minRange, maxRange + 1) + random(minRange, maxRange + 1) + random(1, maxRange + 1)) / 3;
    }
    else
    {
      result = (random(minRange, maxRange + 1) + random(minRange, maxRange + 1) + random(minRange, 0)) / 3;
    }
  }
  return result;
}

void updateLightsFromGauge()
{
  if (gauge_value <= GAUGE_DEEP_MIN_THRESHOLD)
  {
    redLedEndFlash(false);
    blueLedEndFlash(true);
  }
  else if (gauge_value <= GAUGE_MIN_THRESHOLD)
  {
    blueLedStartFlash();
    redLedEndFlash(false);
  }
  else if (gauge_value >= GAUGE_DEEP_MAX_THRESHOLD)
  {
    blueLedEndFlash(false);
    redLedEndFlash(true);
  }
  else if (gauge_value >= GAUGE_MAX_THRESHOLD)
  {
    redLedStartFlash();
    blueLedEndFlash(false);
  }
  else
  {
    blueLedEndFlash(false);
    redLedEndFlash(false);
  }
}

void initializeMeasure()
{
  state = STATE_MEASURE;
  greenLedOn();
  yellowLedEndFlash(false);
  calculateInitialGaugeValue();
  updateLightsFromGauge();
}


void process_synch_state()
{
  int knob_one_delta = abs(analogRead(KNOB_ONE) - knob_one_target);
  int knob_two_delta = abs(analogRead(KNOB_TWO) - knob_two_target);
  if ((knob_one_delta + knob_two_delta) < SYNCH_TARGET)
  {
    initializeMeasure();
  }
  else
  {
    int duration = 0;
    knob_one_delta -= SYNCH_TARGET / 2;
    if (knob_one_delta < 0)
    {
      knob_one_delta = 0;
    }
    knob_two_delta -= SYNCH_TARGET / 2;
    if (knob_two_delta < 0)
    {
      knob_two_delta = 0;
    }
    if (knob_one_delta > (2 * SYNCH_TARGET))
    {
      duration = 2 * 2 * SYNCH_TARGET + ((knob_one_delta - (2 * SYNCH_TARGET)));
    }
    else
    {
      duration = 2 * knob_one_delta;
    }
    if (knob_two_delta > (2 * SYNCH_TARGET))
    {
      duration += 2 * 2 * SYNCH_TARGET + ((knob_two_delta - (2 * SYNCH_TARGET)));
    }
    else
    {
      duration += 2 * knob_two_delta;
    }

    yellowLedFlash((duration / 2) + 150);
  }
}

void debugDisplay(int delta)
{
  //We want to print desired state, current state.
  // Current value and delta.
  byte curr_state = 2;
  if (gauge_value <= GAUGE_DEEP_MIN_THRESHOLD)
  {
    curr_state = 0;
  }
  else if (gauge_value <= GAUGE_MIN_THRESHOLD)
  {
    curr_state = 1;
  }
  else if (gauge_value >= GAUGE_DEEP_MAX_THRESHOLD)
  {
    curr_state = 4;
  }
  else if (gauge_value >= GAUGE_MAX_THRESHOLD)
  {
    curr_state = 3;
  }
}


void process_measure_state()
{ 
  //Capture and trigger reset if necessary here...
  int knob_two_value = analogRead(KNOB_TWO);
  if ((knob_two_value > 1010) && (measure_reset_started == false))
  {
      measure_reset_started = true;
  }
  else if ((knob_two_value < 10) && (synch_reset_started == false))
  {
    synch_reset_started = true;    
  }
  if ((measure_reset_started == true) && (knob_two_value < 800))
  {
    measure_reset_started = false;
    int knob_one_value = analogRead(KNOB_ONE);
    gauge_bias = convertVoltRangeToGaugeBias(knob_one_value);
    initializeMeasure();
  }
  if ((synch_reset_started == true) && (knob_two_value > 200))
  {
    synch_reset_started = false;
    initializeSynch();
  }
  unsigned long elapsed_time = current_time - gauge_last_move;
  if (elapsed_time > gauge_move_duration)
  {
    updateGaugeMoveDuration();
    int delta = generateGaugeDelta();
    gauge_value = gauge_value + delta;
    gauge_needle.write(gauge_value);
    updateLightsFromGauge();
  }
  redLedFlash(CONSTANT_FLASH_DURATION);
  blueLedFlash(CONSTANT_FLASH_DURATION);
}

void setup() {
  // put your setup code here, to run once:
  Serial.begin(9600);
  current_time = millis();
  randomSeed(analogRead(RANDOM_SEED));
  pinMode(RED_LED, OUTPUT);
  pinMode(BLUE_LED, OUTPUT);
  pinMode(YELLOW_LED, OUTPUT);
  pinMode(GREEN_LED, OUTPUT);
  gauge_needle.attach(GAUGE_NEEDLE);
  gauge_needle.write(90);
  digitalWrite(RED_LED, LOW);
  digitalWrite(BLUE_LED, LOW);
  digitalWrite(GREEN_LED, LOW);
  digitalWrite(YELLOW_LED, LOW);
  led_states = 0;
  measure_reset_started = false;
  synch_reset_started = false;
  initializeSynch();
}

void loop() {
  // put your main code here, to run repeatedly:
  current_time = millis();
  int knob_two_value;
  switch(state)
  {
    case STATE_SYNCH:
      process_synch_state();
      break;
    case STATE_MEASURE:
      process_measure_state();
      break;
    default:
      break;
  };
}
